/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.configuration;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.lang.reflect.InvocationTargetException;

import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.topology.generated.Container;
import org.ow2.petals.topology.generated.Domain;
import org.ow2.petals.topology.generated.DomainMode;
import org.ow2.petals.topology.generated.JmxService;
import org.ow2.petals.topology.generated.Topology;
import org.ow2.petals.topology.generated.TransportService;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;

/**
 * Utility class for unit test about the configuration of a Petals ESB node.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ConfigurationTestUtils {

    public final static String DOMAIN_NAME = "PEtALS";

    public final static String DOMAIN_DESCRIPTION = "A sample domain configuration";

    public final static DomainMode DOMAIN_MODE = DomainMode.STATIC;

    public final static String CONTAINER_NAME_0 = "sample-0";

    public final static String CONTAINER_DESCRIPTION_0 = "description of the container 0";

    public final static int CONTAINER_TCP_PORT_0 = 7800;

    public final static String CONTAINER_HOST = "localhost";

    public final static String CONTAINER_JMX_USER = "petals";

    public final static String CONTAINER_JMX_PWD = "petals";

    public final static String TOPOLOGY_PASSPHRASE = "topo-passphrase";

    private ConfigurationTestUtils() {
        // No constructor in utility class
    }

    public static Topology createBasicSampleTopology() {
        return createBasicSampleTopology(DOMAIN_NAME, CONTAINER_NAME_0);
    }

    /**
     * Create a sample topology with only required parameters
     * 
     * @return A sample topology with only required parameters
     */
    public static Topology createBasicSampleTopology(final String domainName, final String containerName) {
        final Topology topology = new Topology();
        final Domain domain = new Domain();
        topology.setDomain(domain);

        domain.setName(domainName);
        domain.setDescription(DOMAIN_DESCRIPTION);

        domain.setMode(DOMAIN_MODE);

        final Container container0 = new Container();
        domain.getContainer().add(container0);
        container0.setName(containerName);
        container0.setDescription(CONTAINER_DESCRIPTION_0);
        container0.setHost(CONTAINER_HOST);
        container0.setUser(CONTAINER_JMX_USER);
        container0.setPassword(CONTAINER_JMX_PWD);

        final JmxService jmxService0 = new JmxService();
        container0.setJmxService(jmxService0);

        final TransportService transportService0 = new TransportService();
        transportService0.setTcpPort(CONTAINER_TCP_PORT_0);
        container0.setTransportService(transportService0);

        return topology;
    }

    /**
     * Assert a basic sample topology, for example created by {@link #createBasicSampleTopology()}
     * 
     * @param topology
     *            The topology to check
     */
    public static void assertBasicSampleTopology(final Topology topology) {

        assertNotNull("Topology is null", topology);

        final Domain domain = topology.getDomain();
        assertNotNull("Domain is null", domain);
        assertEquals("Unexpected domain name", ConfigurationTestUtils.DOMAIN_NAME, domain.getName());
        assertEquals("Unexpected domain description", ConfigurationTestUtils.DOMAIN_DESCRIPTION,
                domain.getDescription());

        assertEquals("Unexpected domain mode", ConfigurationTestUtils.DOMAIN_MODE, domain.getMode());

        assertNotNull("Container list null", domain.getContainer());
        assertEquals("Unexpected number of container", 1, domain.getContainer().size());
        final Container container = domain.getContainer().get(0);
        assertEquals("Unexpected container name", ConfigurationTestUtils.CONTAINER_NAME_0, container.getName());
        assertEquals("Unexpected container description", ConfigurationTestUtils.CONTAINER_DESCRIPTION_0,
                container.getDescription());
        assertEquals("Unexpected container host", ConfigurationTestUtils.CONTAINER_HOST, container.getHost());
        assertEquals("Unexpected container jmx user", ConfigurationTestUtils.CONTAINER_JMX_USER, container.getUser());
        assertEquals("Unexpected container jmx password", ConfigurationTestUtils.CONTAINER_JMX_PWD,
                container.getPassword());

        assertNotNull("JMX service null", container.getJmxService());
        assertNotNull("TCP service null", container.getTransportService());
        assertEquals("Unexpected container TCP port", ConfigurationTestUtils.CONTAINER_TCP_PORT_0, container
                .getTransportService().getTcpPort());

    }

    /**
     * Start a service 'configuration'
     * 
     * @param cfgSvc
     *            The service 'configuration' to start
     * @throws InvocationTargetException
     *             An error occurs during the start. Should not occur
     */
    public static void startConfigurationService(final ConfigurationService cfgSvc) throws InvocationTargetException {
        ReflectionHelper.invokePrivateMethod(cfgSvc, "start", null, null);
    }

    /**
     * Stop a service 'configuration'
     * 
     * @param cfgSvc
     *            The service 'configuration' to stop
     * @throws InvocationTargetException
     *             An error occurs during the stop. Should not occur
     */
    public static void stopConfigurationService(final ConfigurationService cfgSvc) throws InvocationTargetException {
        ReflectionHelper.invokePrivateMethod(cfgSvc, "stop", null, null);
    }
}
