/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.configuration;

import java.io.File;
import java.io.IOException;
import java.util.Properties;

import org.junit.rules.ExternalResource;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.launcher.api.server.conf.ConfigurationProperties;

/**
 * The {@link LocalServerPropertiesBuilder} Rule allows creation of local server properties with their associated resources
 * (directories/files) guaranteed to be deleted when the test method finishes (whether it passes or fails):
 * 
 * <pre>
 * public static class HasLocalServerProperties {
 * 
 *  &#064;Rule
 *  public LocalServerProperties localServerPropertiesBuilder = new LocalServerProperties();
 * 
 *  &#064;Test
 *  public void test...() throws IOException {
 *      final Properties localServerProperties = localServerPropertiesBuilder.createMinimalServerProperties(...);
 *      ...
 *  }
 * }
 * </pre>
 */
public class LocalServerPropertiesBuilder extends ExternalResource {

    /**
     * A Junit rule managing the files and directories required by local server properties (data area, working area,
     * ...)
     */
    private final TemporaryFolder temporaryFolder;

    /**
     * The {@link File} associated to the data area
     */
    private File dataArea = null;

    /**
     * Creates a JUnit rules managing local server properties
     */
    public LocalServerPropertiesBuilder() {
        this.temporaryFolder = new TemporaryFolder();
    }

    @Override
    protected void before() throws Throwable {
        this.temporaryFolder.create();
    }

    @Override
    protected void after() {
        this.temporaryFolder.delete();
    }

    /**
     * <p>
     * Create minimal server properties containing required properties:
     * <ul>
     * <li>{@value ConfigurationProperties#DATA_ROOT_DIRECTORY_PROPERTY_NAME},</li>
     * <li>{@value ConfigurationProperties#CONTAINER_NAME}.</li>
     * </ul>
     * </p>
     * <p>
     * Associated resources are also created.
     * </p>
     * 
     * @param containerName
     *            The container name
     * @return Minimal server properties containing required properties.
     */
    public Properties createMinimalServerProperties(final String containerName) throws IOException {

        this.dataArea = this.temporaryFolder.newFolder("data.root.directory.value");

        final Properties serverProps = new Properties();
        serverProps.put(ConfigurationProperties.DATA_ROOT_DIRECTORY_PROPERTY_NAME, this.dataArea.getAbsolutePath());
        serverProps.put(ConfigurationProperties.CONTAINER_NAME, containerName);

        return serverProps;
    }

    /**
     * @return The path of the data area used in local server properties
     */
    public File getDataArea() {
        return this.dataArea;
    }

}
